﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class NumericalExpression : CodeNode {
    public override int MinimumArity { get { return 1; } }
    public override int MaximumArity { get { return 1; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    protected NumericalExpression(bool deserializing) : base(deserializing) { }
    protected NumericalExpression(NumericalExpression original, Cloner cloner) : base(original, cloner) { }

    public NumericalExpression() : base("NumericalExpression", "A numerical expression.") { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new NumericalExpression(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, IEnumerable<ISymbolicExpressionTreeNode> children) {
      if (children.Count() != 1)
        throw new ArgumentException("Expected 1 child in NumericalExpression.", "children");

      var exprTree = children.Single();
      var symbol = exprTree.Symbol;
      if (!(symbol is Number || symbol is INumericalMethod || symbol is INumericalOperator))
        throw new System.Exception("NumericalExpression was given a child of type " + symbol.GetType() +
            ". The expected child must be of type " + typeof(Number)
                                           + " or " + typeof(INumericalMethod)
                                           + " or " + typeof(INumericalOperator) + ".");

      return Prefix + ((CodeNode)symbol).Interpret(exprTree, exprTree.Subtrees) + Suffix;
    }
  }
}
